@@ -24,8 +24,8 @@
 
 #include "globalconf.h"
 
-void banning_need_update(screen_t *);
+void banning_need_update(void);
 void banning_refresh(void);
 
 #endif
-// vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
+// vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
