@@ -57,30 +57,8 @@ void xutil_lock_mask_get(xcb_connection_t *, xcb_get_modifier_mapping_cookie_t,
                          xcb_key_symbols_t *,
                          uint16_t *, uint16_t *, uint16_t *, uint16_t *);
 
-uint16_t xutil_key_mask_fromstr(const char *, size_t);
+uint16_t xutil_key_mask_fromstr(const char *);
 void xutil_key_mask_tostr(uint16_t, const char **, size_t *);
 
-/* Get the informations about the screen.
- * \param c X connection.
- * \param screen Screen number.
- * \return Screen informations (must not be freed!).
- */
-static inline xcb_screen_t *
-xutil_screen_get(xcb_connection_t *c, int screen)
-{
-    xcb_screen_t *s;
-
-    if(xcb_connection_has_error(c))
-        fatal("X connection invalid");
-
-    s = xcb_aux_get_screen(c, screen);
-
-    assert(s);
-
-    return s;
-}
-
-int xutil_root2screen(xcb_connection_t *, xcb_window_t);
-
 #endif
-// vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
+// vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
