@@ -22,10 +22,10 @@
 #include "config.h"
 
 #include "dbus.h"
+#include <glib.h>
 
 #ifdef WITH_DBUS
 
-#include <ev.h>
 #include <dbus/dbus.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -35,8 +35,8 @@
 
 static DBusConnection *dbus_connection_session = NULL;
 static DBusConnection *dbus_connection_system = NULL;
-ev_io dbusio_ses = { .fd = -1 };
-ev_io dbusio_sys = { .fd = -1 };
+static GSource *session_source = NULL;
+static GSource *system_source = NULL;
 
 static signal_array_t dbus_signals;
 
@@ -45,17 +45,14 @@ static signal_array_t dbus_signals;
  * \param dbusio The D-Bus event watcher
  */
 static void
-a_dbus_cleanup_bus(DBusConnection *dbus_connection, ev_io *dbusio)
+a_dbus_cleanup_bus(DBusConnection *dbus_connection, GSource **source)
 {
     if(!dbus_connection)
         return;
 
-    if(dbusio->fd >= 0)
-    {
-        ev_ref(EV_DEFAULT_UC);
-        ev_io_stop(EV_DEFAULT_UC_ dbusio);
-        dbusio->fd = -1;
-    }
+    if (*source != NULL)
+        g_source_destroy(*source);
+    *source = NULL;
 
     /* This is a shared connection owned by libdbus
      * Do not close it, only unref
@@ -276,7 +273,7 @@ a_dbus_convert_value(lua_State *L, int idx, DBusMessageIter *iter)
                                              type + 1,
                                              &subiter);
 
-            int arraylen = lua_objlen(L, idx + 1);
+            int arraylen = luaA_rawlen(L, idx + 1);
 
             if(arraylen % 2 != 0)
             {
@@ -348,6 +345,7 @@ static void
 a_dbus_process_request(DBusConnection *dbus_connection, DBusMessage *msg)
 {
     const char *interface = dbus_message_get_interface(msg);
+    int old_top = lua_gettop(globalconf.L);
 
     lua_createtable(globalconf.L, 0, 5);
 
@@ -397,8 +395,13 @@ a_dbus_process_request(DBusConnection *dbus_connection, DBusMessage *msg)
         nargs += a_dbus_message_iter(&iter);
 
     if(dbus_message_get_no_reply(msg))
+    {
+        signal_t *sigfound = signal_array_getbyid(&dbus_signals,
+                                                  a_strhash((const unsigned char *) NONULL(interface)));
         /* emit signals */
-        signal_object_emit(globalconf.L, &dbus_signals, NONULL(interface), nargs);
+        if(sigfound)
+            signal_object_emit(globalconf.L, &dbus_signals, NONULL(interface), nargs);
+    }
     else
     {
         signal_t *sig = signal_array_getbyid(&dbus_signals,
@@ -423,8 +426,8 @@ a_dbus_process_request(DBusConnection *dbus_connection, DBusMessage *msg)
             {
                 luaA_warn(globalconf.L,
                           "your D-Bus signal handling method returned wrong number of arguments");
-                /* Remove returned values from the stack */
-                lua_pop(globalconf.L, - n);
+                /* Restore stack */
+                lua_settop(globalconf.L, old_top);
                 return;
             }
 
@@ -434,6 +437,8 @@ a_dbus_process_request(DBusConnection *dbus_connection, DBusMessage *msg)
                 if(!a_dbus_convert_value(globalconf.L, i, &iter))
                 {
                     luaA_warn(globalconf.L, "your D-Bus signal handling method returned bad data");
+                    /* Restore stack */
+                    lua_settop(globalconf.L, old_top);
                     return;
                 }
 
@@ -445,6 +450,8 @@ a_dbus_process_request(DBusConnection *dbus_connection, DBusMessage *msg)
             dbus_message_unref(reply);
         }
     }
+    /* Restore stack */
+    lua_settop(globalconf.L, old_top);
 }
 
 /** Attempt to process all the requests in the D-Bus connection.
@@ -452,7 +459,7 @@ a_dbus_process_request(DBusConnection *dbus_connection, DBusMessage *msg)
  * \param dbusio The D-Bus event watcher
  */
 static void
-a_dbus_process_requests_on_bus(DBusConnection *dbus_connection, ev_io *dbusio)
+a_dbus_process_requests_on_bus(DBusConnection *dbus_connection, GSource **source)
 {
     DBusMessage *msg;
     int nmsg = 0;
@@ -466,7 +473,7 @@ a_dbus_process_requests_on_bus(DBusConnection *dbus_connection, ev_io *dbusio)
 
         if(dbus_message_is_signal(msg, DBUS_INTERFACE_LOCAL, "Disconnected"))
         {
-            a_dbus_cleanup_bus(dbus_connection, dbusio);
+            a_dbus_cleanup_bus(dbus_connection, source);
             dbus_message_unref(msg);
             return;
         }
@@ -486,20 +493,22 @@ a_dbus_process_requests_on_bus(DBusConnection *dbus_connection, ev_io *dbusio)
  * \param w The D-Bus event watcher
  * \param revents (not used)
  */
-static void
-a_dbus_process_requests_session(EV_P_ ev_io *w, int revents)
+static gboolean
+a_dbus_process_requests_session(gpointer data)
 {
-    a_dbus_process_requests_on_bus(dbus_connection_session, w);
+    a_dbus_process_requests_on_bus(dbus_connection_session, &session_source);
+    return TRUE;
 }
 
 /** Foreword D-Bus process system requests on too the correct function.
  * \param w The D-Bus event watcher
  * \param revents (not used)
  */
-static void
-a_dbus_process_requests_system(EV_P_ ev_io *w, int revents)
+static gboolean
+a_dbus_process_requests_system(gpointer data)
 {
-    a_dbus_process_requests_on_bus(dbus_connection_system, w);
+    a_dbus_process_requests_on_bus(dbus_connection_system, &system_source);
+    return TRUE;
 }
 
 /** Attempt to request a D-Bus name
@@ -578,13 +587,12 @@ a_dbus_release_name(DBusConnection *dbus_connection, const char *name)
 /** Attempt to create a new connection to D-Bus
  * \param type The bus type to use when connecting to D-Bus
  * \param type_name The bus type name eg: "session" or "system"
- * \param dbusio The D-Bus event watcher
  * \param cb Function callback to use when processing requests
+ * \param source A new GSource that will be used for watching the dbus connection.
  * \return The requested D-Bus connection on success, NULL on failure.
  */
 static DBusConnection *
-a_dbus_connect(DBusBusType type, const char *type_name,
-               ev_io *dbusio, void *cb)
+a_dbus_connect(DBusBusType type, const char *type_name, GSourceFunc cb, GSource **source)
 {
     int fd;
     DBusConnection *dbus_connection;
@@ -604,16 +612,18 @@ a_dbus_connect(DBusBusType type, const char *type_name,
         dbus_connection_set_exit_on_disconnect(dbus_connection, false);
         if(dbus_connection_get_unix_fd(dbus_connection, &fd))
         {
-            fcntl(fd, F_SETFD, FD_CLOEXEC);
+            GIOChannel *channel = g_io_channel_unix_new(fd);
+            *source = g_io_create_watch(channel, G_IO_IN);
+            g_io_channel_unref(channel);
+            g_source_set_callback(*source, cb, NULL, NULL);
+            g_source_attach(*source, NULL);
 
-            ev_io_init(dbusio, cb, fd, EV_READ);
-            ev_io_start(EV_DEFAULT_UC_ dbusio);
-            ev_unref(EV_DEFAULT_UC);
+            fcntl(fd, F_SETFD, FD_CLOEXEC);
         }
         else
         {
             warn("cannot get D-Bus connection file descriptor");
-            a_dbus_cleanup_bus(dbus_connection, dbusio);
+            a_dbus_cleanup_bus(dbus_connection, source);
         }
     }
 
@@ -626,9 +636,9 @@ void
 a_dbus_init(void)
 {
     dbus_connection_session = a_dbus_connect(DBUS_BUS_SESSION, "session",
-                                             &dbusio_ses, a_dbus_process_requests_session);
+                                             a_dbus_process_requests_session, &session_source);
     dbus_connection_system = a_dbus_connect(DBUS_BUS_SYSTEM, "system",
-                                            &dbusio_sys, a_dbus_process_requests_system);
+                                            a_dbus_process_requests_system, &system_source);
 }
 
 /** Cleanup the D-Bus session and system
@@ -636,27 +646,22 @@ a_dbus_init(void)
 void
 a_dbus_cleanup(void)
 {
-    a_dbus_cleanup_bus(dbus_connection_session, &dbusio_ses);
-    a_dbus_cleanup_bus(dbus_connection_system, &dbusio_sys);
+    a_dbus_cleanup_bus(dbus_connection_session, &session_source);
+    a_dbus_cleanup_bus(dbus_connection_system, &system_source);
 }
 
 /** Retrieve the D-Bus bus by it's name
  * \param name The name of the bus
- * \param len The length of the name variable
  * \return The corresponding D-Bus connection
  */
 static DBusConnection *
-a_dbus_bus_getbyname(const char *name, size_t len)
+a_dbus_bus_getbyname(const char *name)
 {
-    switch(a_tokenize(name, len))
-    {
-      case A_TK_SYSTEM:
+    if(A_STREQ(name, "system"))
         return dbus_connection_system;
-      case A_TK_SESSION:
+    if(A_STREQ(name, "session"))
         return dbus_connection_session;
-      default:
-        return NULL;
-    }
+    return NULL;
 }
 
 /** Register a D-Bus name to receive message from.
@@ -670,10 +675,9 @@ a_dbus_bus_getbyname(const char *name, size_t len)
 static int
 luaA_dbus_request_name(lua_State *L)
 {
-    size_t len;
-    const char *bus = luaL_checklstring(L, 1, &len);
+    const char *bus = luaL_checkstring(L, 1);
     const char *name = luaL_checkstring(L, 2);
-    DBusConnection *dbus_connection = a_dbus_bus_getbyname(bus, len);
+    DBusConnection *dbus_connection = a_dbus_bus_getbyname(bus);
     lua_pushboolean(L, a_dbus_request_name(dbus_connection, name));
     return 1;
 }
@@ -689,10 +693,9 @@ luaA_dbus_request_name(lua_State *L)
 static int
 luaA_dbus_release_name(lua_State *L)
 {
-    size_t len;
-    const char *bus = luaL_checklstring(L, 1, &len);
+    const char *bus = luaL_checkstring(L, 1);
     const char *name = luaL_checkstring(L, 2);
-    DBusConnection *dbus_connection = a_dbus_bus_getbyname(bus, len);
+    DBusConnection *dbus_connection = a_dbus_bus_getbyname(bus);
     lua_pushboolean(L, a_dbus_release_name(dbus_connection, name));
     return 1;
 }
@@ -707,10 +710,9 @@ luaA_dbus_release_name(lua_State *L)
 static int
 luaA_dbus_add_match(lua_State *L)
 {
-    size_t len;
-    const char *bus = luaL_checklstring(L, 1, &len);
+    const char *bus = luaL_checkstring(L, 1);
     const char *name = luaL_checkstring(L, 2);
-    DBusConnection *dbus_connection = a_dbus_bus_getbyname(bus, len);
+    DBusConnection *dbus_connection = a_dbus_bus_getbyname(bus);
 
     if(dbus_connection)
     {
@@ -732,10 +734,9 @@ luaA_dbus_add_match(lua_State *L)
 static int
 luaA_dbus_remove_match(lua_State *L)
 {
-    size_t len;
-    const char *bus = luaL_checklstring(L, 1, &len);
+    const char *bus = luaL_checkstring(L, 1);
     const char *name = luaL_checkstring(L, 2);
-    DBusConnection *dbus_connection = a_dbus_bus_getbyname(bus, len);
+    DBusConnection *dbus_connection = a_dbus_bus_getbyname(bus);
 
     if(dbus_connection)
     {
@@ -754,7 +755,7 @@ luaA_dbus_remove_match(lua_State *L)
  * \lparam The function to call.
  */
 static int
-luaA_dbus_add_signal(lua_State *L)
+luaA_dbus_connect_signal(lua_State *L)
 {
     const char *name = luaL_checkstring(L, 1);
     luaA_checkfunction(L, 2);
@@ -763,7 +764,10 @@ luaA_dbus_add_signal(lua_State *L)
     if(sig)
         luaA_warn(L, "cannot add signal %s on D-Bus, already existing", name);
     else
-        signal_add(&dbus_signals, name, luaA_object_ref(L, 2));
+    {
+        signal_add(&dbus_signals, name);
+        signal_connect(&dbus_signals, name, luaA_object_ref(L, 2));
+    }
     return 0;
 }
 
@@ -775,24 +779,26 @@ luaA_dbus_add_signal(lua_State *L)
  * \lparam The function to call.
  */
 static int
-luaA_dbus_remove_signal(lua_State *L)
+luaA_dbus_disconnect_signal(lua_State *L)
 {
     const char *name = luaL_checkstring(L, 1);
     luaA_checkfunction(L, 2);
     const void *func = lua_topointer(L, 2);
-    signal_remove(&dbus_signals, name, func);
-    luaA_object_unref(L, (void *) func);
+    signal_disconnect(&dbus_signals, name, func);
+    luaA_object_unref(L, func);
     return 0;
 }
 
-const struct luaL_reg awesome_dbus_lib[] =
+const struct luaL_Reg awesome_dbus_lib[] =
 {
     { "request_name", luaA_dbus_request_name },
     { "release_name", luaA_dbus_release_name },
     { "add_match", luaA_dbus_add_match },
     { "remove_match", luaA_dbus_remove_match },
-    { "add_signal", luaA_dbus_add_signal },
-    { "remove_signal", luaA_dbus_remove_signal },
+    { "connect_signal", luaA_dbus_connect_signal },
+    { "disconnect_signal", luaA_dbus_disconnect_signal },
+    { "__index", luaA_default_index },
+    { "__newindex", luaA_default_newindex },
     { NULL, NULL }
 };
 
@@ -811,4 +817,4 @@ a_dbus_cleanup(void)
 }
 
 #endif
-// vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
+// vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
