@@ -21,6 +21,7 @@
 
 #include <xcb/xcb.h>
 #include <xcb/randr.h>
+#include <xcb/shape.h>
 #include <xcb/xcb_atom.h>
 #include <xcb/xcb_icccm.h>
 #include <xcb/xcb_event.h>
@@ -195,6 +196,16 @@ event_handle_button(xcb_button_press_event_t *ev)
         lua_pop(globalconf.L, 1);
         /* check if any button object matches */
         event_button_callback(ev, &drawin->buttons, -1, 1, NULL);
+        /* Either we are receiving this due to ButtonPress/Release on the root
+         * window or because we grabbed the button on the window. In the later
+         * case we have to call AllowEvents.
+         * Use AsyncPointer instead of ReplayPointer so that the event is
+         * "eaten" instead of being handled again on the root window.
+         */
+        if(ev->child == XCB_NONE)
+            xcb_allow_events(globalconf.connection,
+                             XCB_ALLOW_ASYNC_POINTER,
+                             XCB_CURRENT_TIME);
     }
     else if((c = client_getbyframewin(ev->event)))
     {
@@ -285,23 +296,60 @@ event_handle_configurerequest(xcb_configure_request_event_t *ev)
     if((c = client_getbywin(ev->window)))
     {
         area_t geometry = c->geometry;
+        int16_t diff_w = 0, diff_h = 0, diff_border = 0;
 
         if(ev->value_mask & XCB_CONFIG_WINDOW_X)
+        {
             geometry.x = ev->x;
+            /* The ConfigureRequest specifies the position of the outer corner of the client window, we want the frame */
+            geometry.x -= c->border_width;
+        }
         if(ev->value_mask & XCB_CONFIG_WINDOW_Y)
+        {
             geometry.y = ev->y;
+            /* The ConfigureRequest specifies the position of the outer corner of the client window, we want the frame */
+            geometry.y -= c->border_width;
+        }
         if(ev->value_mask & XCB_CONFIG_WINDOW_WIDTH)
+        {
+            uint16_t old_w = geometry.width;
             geometry.width = ev->width;
+            /* The ConfigureRequest specifies the size of the client window, we want the frame */
+            geometry.width += c->titlebar[CLIENT_TITLEBAR_LEFT].size;
+            geometry.width += c->titlebar[CLIENT_TITLEBAR_RIGHT].size;
+            diff_w = geometry.width - old_w;
+        }
         if(ev->value_mask & XCB_CONFIG_WINDOW_HEIGHT)
+        {
+            uint16_t old_h = geometry.height;
             geometry.height = ev->height;
-
+            /* The ConfigureRequest specifies the size of the client window, we want the frame */
+            geometry.height += c->titlebar[CLIENT_TITLEBAR_TOP].size;
+            geometry.height += c->titlebar[CLIENT_TITLEBAR_BOTTOM].size;
+            diff_h = geometry.height - old_h;
+        }
         if(ev->value_mask & XCB_CONFIG_WINDOW_BORDER_WIDTH)
         {
+            diff_border = ev->border_width - c->border_width;
+            diff_h += diff_border;
+            diff_w += diff_border;
+
             luaA_object_push(globalconf.L, c);
             window_set_border_width(globalconf.L, -1, ev->border_width);
             lua_pop(globalconf.L, 1);
         }
 
+        /* If the client resizes without moving itself, apply window gravity */
+        if(c->size_hints.flags & XCB_ICCCM_SIZE_HINT_P_WIN_GRAVITY)
+        {
+            int16_t diff_x = 0, diff_y = 0;
+            xwindow_translate_for_gravity(c->size_hints.win_gravity, diff_border, diff_border, diff_w, diff_h, &diff_x, &diff_y);
+            if(!(ev->value_mask & XCB_CONFIG_WINDOW_X))
+                geometry.x += diff_x;
+            if(!(ev->value_mask & XCB_CONFIG_WINDOW_Y))
+                geometry.y += diff_y;
+        }
+
         if(!client_resize(c, geometry, false))
             /* ICCCM 4.1.5 / 4.2.3, if nothing was changed, send an event saying so */
             client_send_configure(c);
@@ -517,7 +565,7 @@ event_handle_expose(xcb_expose_event_t *ev)
                                       ev->x, ev->y,
                                       ev->width, ev->height);
     if ((client = client_getbyframewin(ev->window)))
-        client_refresh(client);
+        client_refresh_partial(client, ev->x, ev->y, ev->width, ev->height);
 }
 
 /** The key press event handler.
@@ -657,6 +705,24 @@ event_handle_randr_screen_change_notify(xcb_randr_screen_change_notify_event_t *
     awesome_restart();
 }
 
+/** The shape notify event handler.
+ * \param ev The event.
+ */
+static void
+event_handle_shape_notify(xcb_shape_notify_event_t *ev)
+{
+    client_t *c = client_getbywin(ev->affected_window);
+    if (c)
+    {
+        luaA_object_push(globalconf.L, c);
+        if (ev->shape_kind == XCB_SHAPE_SK_BOUNDING)
+            luaA_object_emit_signal(globalconf.L, -1, "property::shape_client_bounding", 0);
+        if (ev->shape_kind == XCB_SHAPE_SK_CLIP)
+            luaA_object_emit_signal(globalconf.L, -1, "property::shape_client_clip", 0);
+        lua_pop(globalconf.L, 1);
+    }
+}
+
 /** The client message event handler.
  * \param ev The event.
  */
@@ -800,6 +866,7 @@ void event_handle(xcb_generic_event_t *event)
     }
 
     static uint8_t randr_screen_change_notify = 0;
+    static uint8_t shape_notify = 0;
 
     if(randr_screen_change_notify == 0)
     {
@@ -809,9 +876,19 @@ void event_handle(xcb_generic_event_t *event)
         if(randr_query->present)
             randr_screen_change_notify = randr_query->first_event + XCB_RANDR_SCREEN_CHANGE_NOTIFY;
     }
+    if(shape_notify == 0)
+    {
+        /* check for shape extension */
+        const xcb_query_extension_reply_t *shape_query;
+        shape_query = xcb_get_extension_data(globalconf.connection, &xcb_shape_id);
+        if(shape_query->present)
+            shape_notify = shape_query->first_event + XCB_SHAPE_NOTIFY;
+    }
 
     if (response_type == randr_screen_change_notify)
         event_handle_randr_screen_change_notify((void *) event);
+    if (response_type == shape_notify)
+        event_handle_shape_notify((void *) event);
 }
 
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
