@@ -1,7 +1,7 @@
 /*
  * event.h - event handlers header
  *
- * Copyright © 2007-2008 Julien Danjou <julien@danjou.info>
+ * Copyright © 2007-2009 Julien Danjou <julien@danjou.info>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,21 +22,22 @@
 #ifndef AWESOME_EVENT_H
 #define AWESOME_EVENT_H
 
-#include "systray.h"
-#include "wibox.h"
-#include "client.h"
+#include "objects/client.h"
+
+/* luaa.c */
+void luaA_emit_refresh(void);
 
 static inline int
 awesome_refresh(void)
 {
+    luaA_emit_refresh();
     banning_refresh();
-    wibox_refresh();
-    systray_refresh();
-    client_stack_refresh();
+    stack_refresh();
+    client_focus_refresh();
     return xcb_flush(globalconf.connection);
 }
 
 void event_handle(xcb_generic_event_t *event);
 
 #endif
-// vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
+// vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
