@@ -26,11 +26,9 @@
 #include <xcb/xcb_atom.h>
 
 #include "ewmh.h"
-#include "tag.h"
+#include "objects/tag.h"
 #include "screen.h"
-#include "client.h"
-#include "widget.h"
-#include "wibox.h"
+#include "objects/client.h"
 #include "luaa.h"
 #include "common/atoms.h"
 #include "common/buffer.h"
@@ -40,25 +38,81 @@
 #define _NET_WM_STATE_ADD 1
 #define _NET_WM_STATE_TOGGLE 2
 
-/** Update the desktop geometry.
- * \param phys_screen The physical screen id.
+/** Update client EWMH hints.
+ * \param c The client.
  */
-static void
-ewmh_update_desktop_geometry(int phys_screen)
+static int
+ewmh_client_update_hints(lua_State *L)
 {
-    area_t geom = screen_area_get(&globalconf.screens.tab[phys_screen], false);
-    uint32_t sizes[] = { geom.width, geom.height };
+    client_t *c = luaA_checkudata(L, 1, &client_class);
+    xcb_atom_t state[10]; /* number of defined state atoms */
+    int i = 0;
+
+    if(c->modal)
+        state[i++] = _NET_WM_STATE_MODAL;
+    if(c->fullscreen)
+        state[i++] = _NET_WM_STATE_FULLSCREEN;
+    if(c->maximized_vertical)
+        state[i++] = _NET_WM_STATE_MAXIMIZED_VERT;
+    if(c->maximized_horizontal)
+        state[i++] = _NET_WM_STATE_MAXIMIZED_HORZ;
+    if(c->sticky)
+        state[i++] = _NET_WM_STATE_STICKY;
+    if(c->skip_taskbar)
+        state[i++] = _NET_WM_STATE_SKIP_TASKBAR;
+    if(c->above)
+        state[i++] = _NET_WM_STATE_ABOVE;
+    if(c->below)
+        state[i++] = _NET_WM_STATE_BELOW;
+    if(c->minimized)
+        state[i++] = _NET_WM_STATE_HIDDEN;
+    if(c->urgent)
+        state[i++] = _NET_WM_STATE_DEMANDS_ATTENTION;
 
     xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
-                        xutil_screen_get(globalconf.connection, phys_screen)->root,
-                        _NET_DESKTOP_GEOMETRY, XCB_ATOM_CARDINAL, 32, countof(sizes), sizes);
+                        c->window, _NET_WM_STATE, XCB_ATOM_ATOM, 32, i, state);
+
+    return 0;
+}
+
+static int
+ewmh_update_net_active_window(lua_State *L)
+{
+    xcb_window_t win;
+
+    if(globalconf.focus.client)
+        win = globalconf.focus.client->window;
+    else
+        win = XCB_NONE;
+
+    xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
+			globalconf.screen->root,
+			_NET_ACTIVE_WINDOW, XCB_ATOM_WINDOW, 32, 1, &win);
+
+    return 0;
+}
+
+static int
+ewmh_update_net_client_list(lua_State *L)
+{
+    xcb_window_t *wins = p_alloca(xcb_window_t, globalconf.clients.len);
+
+    int n = 0;
+    foreach(client, globalconf.clients)
+        wins[n++] = (*client)->window;
+
+    xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
+                        globalconf.screen->root,
+                        _NET_CLIENT_LIST, XCB_ATOM_WINDOW, 32, n, wins);
+
+    return 0;
 }
 
 void
-ewmh_init(int phys_screen)
+ewmh_init(void)
 {
     xcb_window_t father;
-    xcb_screen_t *xscreen = xutil_screen_get(globalconf.connection, phys_screen);
+    xcb_screen_t *xscreen = globalconf.screen;
     xcb_atom_t atom[] =
     {
         _NET_SUPPORTED,
@@ -70,7 +124,6 @@ ewmh_init(int phys_screen)
         _NET_CURRENT_DESKTOP,
         _NET_DESKTOP_NAMES,
         _NET_ACTIVE_WINDOW,
-        _NET_DESKTOP_GEOMETRY,
         _NET_CLOSE_WINDOW,
         _NET_WM_NAME,
         _NET_WM_STRUT_PARTIAL,
@@ -135,100 +188,78 @@ ewmh_init(int phys_screen)
     xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
                         father, _NET_WM_PID, XCB_ATOM_CARDINAL, 32, 1, &i);
 
-    ewmh_update_desktop_geometry(phys_screen);
-}
 
-void
-ewmh_update_net_client_list(int phys_screen)
-{
-    xcb_window_t *wins = p_alloca(xcb_window_t, globalconf.clients.len);
-
-    int n = 0;
-    foreach(_c, globalconf.clients)
-    {
-        client_t *c = *_c;
-        if(c->phys_screen == phys_screen)
-            wins[n++] = c->window;
-    }
-
-    xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
-			xutil_screen_get(globalconf.connection, phys_screen)->root,
-			_NET_CLIENT_LIST, XCB_ATOM_WINDOW, 32, n, wins);
+    luaA_class_connect_signal(globalconf.L, &client_class, "focus", ewmh_update_net_active_window);
+    luaA_class_connect_signal(globalconf.L, &client_class, "unfocus", ewmh_update_net_active_window);
+    luaA_class_connect_signal(globalconf.L, &client_class, "manage", ewmh_update_net_client_list);
+    luaA_class_connect_signal(globalconf.L, &client_class, "unmanage", ewmh_update_net_client_list);
+    luaA_class_connect_signal(globalconf.L, &client_class, "property::modal" , ewmh_client_update_hints);
+    luaA_class_connect_signal(globalconf.L, &client_class, "property::fullscreen" , ewmh_client_update_hints);
+    luaA_class_connect_signal(globalconf.L, &client_class, "property::maximized_horizontal" , ewmh_client_update_hints);
+    luaA_class_connect_signal(globalconf.L, &client_class, "property::maximized_vertical" , ewmh_client_update_hints);
+    luaA_class_connect_signal(globalconf.L, &client_class, "property::sticky" , ewmh_client_update_hints);
+    luaA_class_connect_signal(globalconf.L, &client_class, "property::skip_taskbar" , ewmh_client_update_hints);
+    luaA_class_connect_signal(globalconf.L, &client_class, "property::above" , ewmh_client_update_hints);
+    luaA_class_connect_signal(globalconf.L, &client_class, "property::below" , ewmh_client_update_hints);
+    luaA_class_connect_signal(globalconf.L, &client_class, "property::minimized" , ewmh_client_update_hints);
+    luaA_class_connect_signal(globalconf.L, &client_class, "property::urgent" , ewmh_client_update_hints);
 }
 
 /** Set the client list in stacking order, bottom to top.
- * \param phys_screen The physical screen id.
  */
 void
-ewmh_update_net_client_list_stacking(int phys_screen)
+ewmh_update_net_client_list_stacking(void)
 {
     int n = 0;
     xcb_window_t *wins = p_alloca(xcb_window_t, globalconf.stack.len);
 
     foreach(client, globalconf.stack)
-        if((*client)->phys_screen == phys_screen)
-            wins[n++] = (*client)->window;
+        wins[n++] = (*client)->window;
 
     xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
-			xutil_screen_get(globalconf.connection, phys_screen)->root,
+			globalconf.screen->root,
 			_NET_CLIENT_LIST_STACKING, XCB_ATOM_WINDOW, 32, n, wins);
 }
 
 void
-ewmh_update_net_numbers_of_desktop(int phys_screen)
+ewmh_update_net_numbers_of_desktop(void)
 {
-    uint32_t count = globalconf.screens.tab[phys_screen].tags.len;
+    uint32_t count = globalconf.tags.len;
 
     xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
-			xutil_screen_get(globalconf.connection, phys_screen)->root,
+			globalconf.screen->root,
 			_NET_NUMBER_OF_DESKTOPS, XCB_ATOM_CARDINAL, 32, 1, &count);
 }
 
 void
-ewmh_update_net_current_desktop(int phys_screen)
+ewmh_update_net_current_desktop(void)
 {
-    uint32_t idx = tags_get_first_selected_index(&globalconf.screens.tab[phys_screen]);
+    uint32_t idx = tags_get_first_selected_index();
 
     xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
-                        xutil_screen_get(globalconf.connection, phys_screen)->root,
+                        globalconf.screen->root,
                         _NET_CURRENT_DESKTOP, XCB_ATOM_CARDINAL, 32, 1, &idx);
 }
 
 void
-ewmh_update_net_desktop_names(int phys_screen)
+ewmh_update_net_desktop_names(void)
 {
     buffer_t buf;
 
     buffer_inita(&buf, BUFSIZ);
 
-    foreach(tag, globalconf.screens.tab[phys_screen].tags)
+    foreach(tag, globalconf.tags)
     {
         buffer_adds(&buf, tag_get_name(*tag));
         buffer_addc(&buf, '\0');
     }
 
     xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
-			xutil_screen_get(globalconf.connection, phys_screen)->root,
+			globalconf.screen->root,
 			_NET_DESKTOP_NAMES, UTF8_STRING, 8, buf.len, buf.s);
     buffer_wipe(&buf);
 }
 
-void
-ewmh_update_net_active_window(int phys_screen)
-{
-    xcb_window_t win;
-
-    if(globalconf.screen_focus->client_focus
-       && globalconf.screen_focus->client_focus->phys_screen == phys_screen)
-        win = globalconf.screen_focus->client_focus->window;
-    else
-        win = XCB_NONE;
-
-    xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
-			xutil_screen_get(globalconf.connection, phys_screen)->root,
-			_NET_ACTIVE_WINDOW, XCB_ATOM_WINDOW, 32, 1, &win);
-}
-
 static void
 ewmh_process_state_atom(client_t *c, xcb_atom_t state, int set)
 {
@@ -246,20 +277,11 @@ ewmh_process_state_atom(client_t *c, xcb_atom_t state, int set)
     else if(state == _NET_WM_STATE_SKIP_TASKBAR)
     {
         if(set == _NET_WM_STATE_REMOVE)
-        {
             client_set_skip_taskbar(globalconf.L, -1, false);
-            ewmh_client_update_hints(c);
-        }
         else if(set == _NET_WM_STATE_ADD)
-        {
             client_set_skip_taskbar(globalconf.L, -1, true);
-            ewmh_client_update_hints(c);
-        }
         else if(set == _NET_WM_STATE_TOGGLE)
-        {
             client_set_skip_taskbar(globalconf.L, -1, !c->skip_taskbar);
-            ewmh_client_update_hints(c);
-        }
     }
     else if(state == _NET_WM_STATE_FULLSCREEN)
     {
@@ -337,20 +359,43 @@ ewmh_process_state_atom(client_t *c, xcb_atom_t state, int set)
     lua_pop(globalconf.L, 1);
 }
 
+static void
+ewmh_process_desktop(client_t *c, uint32_t desktop)
+{
+    int idx = desktop;
+    if(desktop == 0xffffffff)
+    {
+        luaA_object_push(globalconf.L, c);
+        lua_pushnil(globalconf.L);
+        luaA_object_emit_signal(globalconf.L, -2, "request::tag", 1);
+        /* Pop the client, arguments are already popped */
+        lua_pop(globalconf.L, 1);
+    }
+    else if (idx >= 0 && idx < globalconf.tags.len)
+    {
+        luaA_object_push(globalconf.L, c);
+        luaA_object_push(globalconf.L, globalconf.tags.tab[idx]);
+        luaA_object_emit_signal(globalconf.L, -2, "request::tag", 1);
+        /* Pop the client, arguments are already popped */
+        lua_pop(globalconf.L, 1);
+    }
+}
+
 int
 ewmh_process_client_message(xcb_client_message_event_t *ev)
 {
     client_t *c;
-    int screen;
 
     if(ev->type == _NET_CURRENT_DESKTOP)
-        for(screen = 0;
-            screen < xcb_setup_roots_length(xcb_get_setup(globalconf.connection));
-            screen++)
+    {
+        int idx = ev->data.data32[0];
+        if (idx >= 0 && idx < globalconf.tags.len)
         {
-            if(ev->window == xutil_screen_get(globalconf.connection, screen)->root)
-                tag_view_only_byindex(&globalconf.screens.tab[screen], ev->data.data32[0]);
+            luaA_object_push(globalconf.L, globalconf.tags.tab[idx]);
+            luaA_object_emit_signal(globalconf.L, -1, "request::select", 0);
+            lua_pop(globalconf.L, 1);
         }
+    }
     else if(ev->type == _NET_CLOSE_WINDOW)
     {
         if((c = client_getbywin(ev->window)))
@@ -360,19 +405,7 @@ ewmh_process_client_message(xcb_client_message_event_t *ev)
     {
         if((c = client_getbywin(ev->window)))
         {
-            tag_array_t *tags = &c->screen->tags;
-
-            if(ev->data.data32[0] == 0xffffffff)
-                c->sticky = true;
-            else
-                for(int i = 0; i < tags->len; i++)
-                    if((int)ev->data.data32[0] == i)
-                    {
-                        luaA_object_push(globalconf.L, tags->tab[i]);
-                        tag_client(c);
-                    }
-                    else
-                        untag_client(c, tags->tab[i]);
+            ewmh_process_desktop(c, ev->data.data32[0]);
         }
     }
     else if(ev->type == _NET_WM_STATE)
@@ -387,47 +420,17 @@ ewmh_process_client_message(xcb_client_message_event_t *ev)
     }
     else if(ev->type == _NET_ACTIVE_WINDOW)
     {
-        if((c = client_getbywin(ev->window)))
-            client_focus(c);
+        if((c = client_getbywin(ev->window))) {
+            luaA_object_push(globalconf.L, c);
+            lua_pushstring(globalconf.L,"ewmh");
+            luaA_object_emit_signal(globalconf.L, -2, "request::activate", 1);
+            lua_pop(globalconf.L, 1);
+        }
     }
 
     return 0;
 }
 
-/** Update client EWMH hints.
- * \param c The client.
- */
-void
-ewmh_client_update_hints(client_t *c)
-{
-    xcb_atom_t state[10]; /* number of defined state atoms */
-    int i = 0;
-
-    if(c->modal)
-        state[i++] = _NET_WM_STATE_MODAL;
-    if(c->fullscreen)
-        state[i++] = _NET_WM_STATE_FULLSCREEN;
-    if(c->maximized_vertical)
-        state[i++] = _NET_WM_STATE_MAXIMIZED_VERT;
-    if(c->maximized_horizontal)
-        state[i++] = _NET_WM_STATE_MAXIMIZED_HORZ;
-    if(c->sticky)
-        state[i++] = _NET_WM_STATE_STICKY;
-    if(c->skip_taskbar)
-        state[i++] = _NET_WM_STATE_SKIP_TASKBAR;
-    if(c->above)
-        state[i++] = _NET_WM_STATE_ABOVE;
-    if(c->below)
-        state[i++] = _NET_WM_STATE_BELOW;
-    if(c->minimized)
-        state[i++] = _NET_WM_STATE_HIDDEN;
-    if(c->urgent)
-        state[i++] = _NET_WM_STATE_DEMANDS_ATTENTION;
-
-    xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
-                        c->window, _NET_WM_STATE, XCB_ATOM_ATOM, 32, i, state);
-}
-
 /** Update the client active desktop.
  * This is "wrong" since it can be on several tags, but EWMH has a strict view
  * of desktop system so just take the first tag.
@@ -437,10 +440,9 @@ void
 ewmh_client_update_desktop(client_t *c)
 {
     int i;
-    tag_array_t *tags = &c->screen->tags;
 
-    for(i = 0; i < tags->len; i++)
-        if(is_client_tagged(c, tags->tab[i]))
+    for(i = 0; i < globalconf.tags.len; i++)
+        if(is_client_tagged(c, globalconf.tags.tab[i]))
         {
             xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
                                 c->window, _NET_WM_DESKTOP, XCB_ATOM_CARDINAL, 32, 1, &i);
@@ -457,24 +459,38 @@ ewmh_client_update_desktop(client_t *c)
 void
 ewmh_update_strut(xcb_window_t window, strut_t *strut)
 {
-    const uint32_t state[] =
+    if(window)
     {
-        strut->left,
-        strut->right,
-        strut->top,
-        strut->bottom,
-        strut->left_start_y,
-        strut->left_end_y,
-        strut->right_start_y,
-        strut->right_end_y,
-        strut->top_start_x,
-        strut->top_end_x,
-        strut->bottom_start_x,
-        strut->bottom_end_x
-    };
+        const uint32_t state[] =
+        {
+            strut->left,
+            strut->right,
+            strut->top,
+            strut->bottom,
+            strut->left_start_y,
+            strut->left_end_y,
+            strut->right_start_y,
+            strut->right_end_y,
+            strut->top_start_x,
+            strut->top_end_x,
+            strut->bottom_start_x,
+            strut->bottom_end_x
+        };
 
+        xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
+                            window, _NET_WM_STRUT_PARTIAL, XCB_ATOM_CARDINAL, 32, countof(state), state);
+    }
+}
+
+/** Update the window type.
+ * \param window The window to update.
+ * \param type The new type to set.
+ */
+void
+ewmh_update_window_type(xcb_window_t window, uint32_t type)
+{
     xcb_change_property(globalconf.connection, XCB_PROP_MODE_REPLACE,
-                        window, _NET_WM_STRUT_PARTIAL, XCB_ATOM_CARDINAL, 32, countof(state), state);
+                        window, _NET_WM_WINDOW_TYPE, XCB_ATOM_ATOM, 32, 1, &type);
 }
 
 void
@@ -482,7 +498,6 @@ ewmh_client_check_hints(client_t *c)
 {
     xcb_atom_t *state;
     void *data = NULL;
-    int desktop;
     xcb_get_property_cookie_t c0, c1, c2;
     xcb_get_property_reply_t *reply;
 
@@ -499,27 +514,7 @@ ewmh_client_check_hints(client_t *c)
     reply = xcb_get_property_reply(globalconf.connection, c0, NULL);
     if(reply && reply->value_len && (data = xcb_get_property_value(reply)))
     {
-        tag_array_t *tags = &c->screen->tags;
-
-        desktop = *(uint32_t *) data;
-        if(desktop == -1)
-            c->sticky = true;
-        else if (desktop >= 0 && desktop < tags->len)
-            for(int i = 0; i < tags->len; i++)
-                if(desktop == i)
-                {
-                    luaA_object_push(globalconf.L, tags->tab[i]);
-                    tag_client(c);
-                }
-                else
-                    untag_client(c, tags->tab[i]);
-        else
-            /* Value out of bounds, just give it the first tag */
-            if (tags->len > 0)
-            {
-                luaA_object_push(globalconf.L, tags->tab[0]);
-                tag_client(c);
-            }
+        ewmh_process_desktop(c, *(uint32_t *) data);
     }
 
     p_delete(&reply);
@@ -563,17 +558,14 @@ ewmh_client_check_hints(client_t *c)
  * \param strut_r (Optional) An existing reply.
  */
 void
-ewmh_process_client_strut(client_t *c, xcb_get_property_reply_t *strut_r)
+ewmh_process_client_strut(client_t *c)
 {
     void *data;
-    xcb_get_property_reply_t *mstrut_r = NULL;
+    xcb_get_property_reply_t *strut_r;
 
-    if(!strut_r)
-    {
-        xcb_get_property_cookie_t strut_q = xcb_get_property_unchecked(globalconf.connection, false, c->window,
-                                                                       _NET_WM_STRUT_PARTIAL, XCB_ATOM_CARDINAL, 0, 12);
-        strut_r = mstrut_r = xcb_get_property_reply(globalconf.connection, strut_q, NULL);
-    }
+    xcb_get_property_cookie_t strut_q = xcb_get_property_unchecked(globalconf.connection, false, c->window,
+                                                                   _NET_WM_STRUT_PARTIAL, XCB_ATOM_CARDINAL, 0, 12);
+    strut_r = xcb_get_property_reply(globalconf.connection, strut_q, NULL);
 
     if(strut_r
        && strut_r->value_len
@@ -607,14 +599,13 @@ ewmh_process_client_strut(client_t *c, xcb_get_property_reply_t *strut_r)
             c->strut.bottom_start_x = strut[10];
             c->strut.bottom_end_x = strut[11];
 
-            hook_property(c, "struts");
             luaA_object_push(globalconf.L, c);
             luaA_object_emit_signal(globalconf.L, -1, "property::struts", 0);
             lua_pop(globalconf.L, 1);
         }
     }
 
-    p_delete(&mstrut_r);
+    p_delete(&strut_r);
 }
 
 /** Send request to get NET_WM_ICON (EWMH)
@@ -628,7 +619,7 @@ ewmh_window_icon_get_unchecked(xcb_window_t w)
                                     _NET_WM_ICON, XCB_ATOM_CARDINAL, 0, UINT32_MAX);
 }
 
-int
+static cairo_surface_t *
 ewmh_window_icon_from_reply(xcb_get_property_reply_t *r)
 {
     uint32_t *data;
@@ -649,20 +640,20 @@ ewmh_window_icon_from_reply(xcb_get_property_reply_t *r)
     if (!data[0] || !data[1] || len > r->length - 2)
         return 0;
 
-    return image_new_from_argb32(data[0], data[1], data + 2);
+    return draw_surface_from_data(data[0], data[1], data + 2);
 }
 
 /** Get NET_WM_ICON.
  * \param cookie The cookie.
  * \return The number of elements on stack.
  */
-int
+cairo_surface_t *
 ewmh_window_icon_get_reply(xcb_get_property_cookie_t cookie)
 {
     xcb_get_property_reply_t *r = xcb_get_property_reply(globalconf.connection, cookie, NULL);
-    int ret = ewmh_window_icon_from_reply(r);
+    cairo_surface_t *surface = ewmh_window_icon_from_reply(r);
     p_delete(&r);
-    return ret;
+    return surface;
 }
 
-// vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
+// vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
