@@ -22,25 +22,24 @@
 #ifndef AWESOME_EWMH_H
 #define AWESOME_EWMH_H
 
+#include <cairo.h>
+
 #include "globalconf.h"
 #include "strut.h"
 
-void ewmh_init(int);
-void ewmh_update_net_client_list(int);
-void ewmh_update_net_numbers_of_desktop(int);
-void ewmh_update_net_current_desktop(int);
-void ewmh_update_net_desktop_names(int);
-void ewmh_update_net_active_window(int);
+void ewmh_init(void);
+void ewmh_update_net_numbers_of_desktop(void);
+void ewmh_update_net_current_desktop(void);
+void ewmh_update_net_desktop_names(void);
 int ewmh_process_client_message(xcb_client_message_event_t *);
-void ewmh_update_net_client_list_stacking(int);
+void ewmh_update_net_client_list_stacking(void);
 void ewmh_client_check_hints(client_t *);
-void ewmh_client_update_hints(client_t *);
 void ewmh_client_update_desktop(client_t *);
-void ewmh_process_client_strut(client_t *, xcb_get_property_reply_t *);
+void ewmh_process_client_strut(client_t *);
 void ewmh_update_strut(xcb_window_t, strut_t *);
+void ewmh_update_window_type(xcb_window_t window, uint32_t type);
 xcb_get_property_cookie_t ewmh_window_icon_get_unchecked(xcb_window_t);
-int ewmh_window_icon_from_reply(xcb_get_property_reply_t *);
-int ewmh_window_icon_get_reply(xcb_get_property_cookie_t);
+cairo_surface_t *ewmh_window_icon_get_reply(xcb_get_property_cookie_t);
 
 #endif
-// vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
+// vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
