@@ -21,11 +21,10 @@
 
 #include "mouse.h"
 #include "screen.h"
-#include "client.h"
+#include "objects/client.h"
 #include "globalconf.h"
-#include "wibox.h"
+#include "objects/drawin.h"
 #include "luaa.h"
-#include "common/tokenize.h"
 #include "common/xutil.h"
 
 /** Get the pointer position.
@@ -46,10 +45,14 @@ mouse_query_pointer(xcb_window_t window, int16_t *x, int16_t *y, xcb_window_t *c
     query_ptr_r = xcb_query_pointer_reply(globalconf.connection, query_ptr_c, NULL);
 
     if(!query_ptr_r || !query_ptr_r->same_screen)
+    {
+        p_delete(&query_ptr_r);
         return false;
+    }
 
     *x = query_ptr_r->win_x;
     *y = query_ptr_r->win_y;
+
     if(mask)
         *mask = query_ptr_r->mask;
     if(child)
@@ -61,7 +64,6 @@ mouse_query_pointer(xcb_window_t window, int16_t *x, int16_t *y, xcb_window_t *c
 }
 
 /** Get the pointer position on the screen.
- * \param s This will be set to the screen the mouse is on.
  * \param x This will be set to the Pointer-x-coordinate relative to window.
  * \param y This will be set to the Pointer-y-coordinate relative to window.
  * \param child This will be set to the window under the pointer.
@@ -69,21 +71,11 @@ mouse_query_pointer(xcb_window_t window, int16_t *x, int16_t *y, xcb_window_t *c
  * \return True on success, false if an error occurred.
  */
 static bool
-mouse_query_pointer_root(screen_t **s, int16_t *x, int16_t *y, xcb_window_t *child, uint16_t *mask)
+mouse_query_pointer_root(int16_t *x, int16_t *y, xcb_window_t *child, uint16_t *mask)
 {
-    for(int screen = 0;
-        screen < xcb_setup_roots_length(xcb_get_setup(globalconf.connection));
-        screen++)
-    {
-        xcb_window_t root = xutil_screen_get(globalconf.connection, screen)->root;
+    xcb_window_t root = globalconf.screen->root;
 
-        if(mouse_query_pointer(root, x, y, child, mask))
-        {
-            *s = &globalconf.screens.tab[screen];
-            return true;
-        }
-    }
-    return false;
+    return mouse_query_pointer(root, x, y, child, mask);
 }
 
 /** Set the pointer position.
@@ -95,7 +87,7 @@ static inline void
 mouse_warp_pointer(xcb_window_t window, int x, int y)
 {
     xcb_warp_pointer(globalconf.connection, XCB_NONE, window,
-                     0, 0, 0, 0, x, y );
+                     0, 0, 0, 0, x, y);
 }
 
 /** Mouse library.
@@ -108,25 +100,28 @@ mouse_warp_pointer(xcb_window_t window, int x, int y)
 static int
 luaA_mouse_index(lua_State *L)
 {
-    size_t len;
-    const char *attr = luaL_checklstring(L, 2, &len);
+    const char *attr = luaL_checkstring(L, 2);
     int16_t mouse_x, mouse_y;
     screen_t *screen;
 
-    switch(a_tokenize(attr, len))
+    /* attr is not "screen"?! */
+    if (A_STRNEQ(attr, "screen"))
+        return luaA_default_index(L);
+
+    if (!mouse_query_pointer_root(&mouse_x, &mouse_y, NULL, NULL))
     {
-      case A_TK_SCREEN:
-        if(!mouse_query_pointer_root(&screen, &mouse_x, &mouse_y, NULL, NULL))
-            return 0;
-
-        screen  = screen_getbycoord(screen, mouse_x, mouse_y);
-
-        lua_pushnumber(L, screen_array_indexof(&globalconf.screens, screen) + 1);
-        break;
-      default:
-        return 0;
+        /* Nothing ever handles mouse.screen being nil. Lying is better than
+         * having lots of lua errors in this case.
+         */
+        if (globalconf.focus.client)
+            lua_pushnumber(L, screen_array_indexof(&globalconf.screens, globalconf.focus.client->screen) + 1);
+        else
+            lua_pushnumber(L, 1);
+        return 1;
     }
 
+    screen = screen_getbycoord(mouse_x, mouse_y);
+    lua_pushnumber(L, screen_array_indexof(&globalconf.screens, screen) + 1);
     return 1;
 }
 
@@ -137,31 +132,20 @@ luaA_mouse_index(lua_State *L)
 static int
 luaA_mouse_newindex(lua_State *L)
 {
-    size_t len;
-    const char *attr = luaL_checklstring(L, 2, &len);
+    const char *attr = luaL_checkstring(L, 2);
     int x, y = 0;
-    xcb_window_t root;
-    int screen, phys_screen;
+    int screen;
 
-    switch(a_tokenize(attr, len))
-    {
-      case A_TK_SCREEN:
-        screen = luaL_checknumber(L, 3) - 1;
-        luaA_checkscreen(screen);
+    if (A_STRNEQ(attr, "screen"))
+        return luaA_default_newindex(L);
 
-        /* we need the physical one to get the root window */
-        phys_screen = screen_virttophys(screen);
-        root = xutil_screen_get(globalconf.connection, phys_screen)->root;
+    screen = luaL_checknumber(L, 3) - 1;
+    luaA_checkscreen(screen);
 
-        x = globalconf.screens.tab[screen].geometry.x;
-        y = globalconf.screens.tab[screen].geometry.y;
-
-        mouse_warp_pointer(root, x, y);
-        break;
-      default:
-        return 0;
-    }
+    x = globalconf.screens.tab[screen].geometry.x;
+    y = globalconf.screens.tab[screen].geometry.y;
 
+    mouse_warp_pointer(globalconf.screen->root, x, y);
     return 0;
 }
 
@@ -206,28 +190,22 @@ luaA_mouse_coords(lua_State *L)
     uint16_t mask;
     int x, y;
     int16_t mouse_x, mouse_y;
-    screen_t *screen;
 
     if(lua_gettop(L) >= 1)
     {
-        xcb_window_t root;
-
         luaA_checktable(L, 1);
         bool ignore_enter_notify = (lua_gettop(L) == 2 && luaA_checkboolean(L, 2));
 
-        if(!mouse_query_pointer_root(&screen, &mouse_x, &mouse_y, NULL, &mask))
+        if(!mouse_query_pointer_root(&mouse_x, &mouse_y, NULL, &mask))
             return 0;
 
         x = luaA_getopt_number(L, 1, "x", mouse_x);
         y = luaA_getopt_number(L, 1, "y", mouse_y);
 
-        root = xutil_screen_get(globalconf.connection,
-                                screen_array_indexof(&globalconf.screens, screen))->root;
-
         if(ignore_enter_notify)
             client_ignore_enterleave_events();
 
-        mouse_warp_pointer(root, x, y);
+        mouse_warp_pointer(globalconf.screen->root, x, y);
 
         if(ignore_enter_notify)
             client_restore_enterleave_events();
@@ -235,7 +213,7 @@ luaA_mouse_coords(lua_State *L)
         lua_pop(L, 1);
     }
 
-    if(!mouse_query_pointer_root(&screen, &mouse_x, &mouse_y, NULL, &mask))
+    if(!mouse_query_pointer_root(&mouse_x, &mouse_y, NULL, &mask))
         return 0;
 
     return luaA_mouse_pushstatus(L, mouse_x, mouse_y, mask);
@@ -250,41 +228,25 @@ luaA_mouse_coords(lua_State *L)
 static int
 luaA_mouse_object_under_pointer(lua_State *L)
 {
-    screen_t *screen;
     int16_t mouse_x, mouse_y;
     xcb_window_t child;
 
-    if(!mouse_query_pointer_root(&screen, &mouse_x, &mouse_y, &child, NULL))
+    if(!mouse_query_pointer_root(&mouse_x, &mouse_y, &child, NULL))
         return 0;
 
-    wibox_t *wibox;
+    drawin_t *drawin;
     client_t *client;
-    if((wibox = wibox_getbywin(child)))
-    {
-        luaA_object_push(L, wibox);
 
-        int16_t x = mouse_x - wibox->geometry.x;
-        int16_t y = mouse_y - wibox->geometry.y;
+    if((drawin = drawin_getbywin(child)))
+        return luaA_object_push(L, drawin);
 
-        widget_t *widget = widget_getbycoords(wibox->orientation, &wibox->widgets,
-                                              wibox->geometry.width,
-                                              wibox->geometry.height,
-                                              &x, &y);
-
-        if(widget)
-        {
-            luaA_object_push(L, widget);
-            return 2;
-        }
-        return 1;
-    }
-    else if((client = client_getbywin(child)))
+    if((client = client_getbyframewin(child)))
         return luaA_object_push(globalconf.L, client);
 
     return 0;
 }
 
-const struct luaL_reg awesome_mouse_methods[] =
+const struct luaL_Reg awesome_mouse_methods[] =
 {
     { "__index", luaA_mouse_index },
     { "__newindex", luaA_mouse_newindex },
@@ -292,9 +254,9 @@ const struct luaL_reg awesome_mouse_methods[] =
     { "object_under_pointer", luaA_mouse_object_under_pointer },
     { NULL, NULL }
 };
-const struct luaL_reg awesome_mouse_meta[] =
+const struct luaL_Reg awesome_mouse_meta[] =
 {
     { NULL, NULL }
 };
 
-// vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
+// vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
