@@ -27,6 +27,8 @@
 void stack_client_remove(client_t *);
 void stack_client_push(client_t *);
 void stack_client_append(client_t *);
+void stack_windows(void);
+void stack_refresh(void);
 
 #endif
-// vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
+// vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:textwidth=80
